# -*- coding: utf-8 -*-
"""This file imports Python modules that registers scaffolders."""
from l2tscaffolder.scaffolders import plaso_sqlite
from l2tscaffolder.scaffolders import timesketch_index
from l2tscaffolder.scaffolders import timesketch_sketch
from l2tscaffolder.scaffolders import turbinia_job_task
