# -*- coding: utf-8 -*-
"""This file imports Python modules that registers definitions."""

from l2tscaffolder.definitions import plaso
from l2tscaffolder.definitions import timesketch
from l2tscaffolder.definitions import turbinia
